!= 金星雲 (Pollack 1993) の光学特性計算プログラム
!
!= optical properties of Venus cloud (1993) calculation program
!
! Authors::   Masanori Onishi
! Version::   $Id: main_dcrtm.f90,v 1.00 onishi$
! Tag Name::  $Name: dcrtm-20170126 $
! Copyright:: Copyright (C) GFD Dennou Club, 2016. All rights reserved.
! License::   See COPYRIGHT[link:../../../COPYRIGHT]

!2017/01/24
!2017/01/25

program initialdata_pollackcloud
  !
  ! <b>Note that Japanese and English are described in parallel.</b>
  !

  ! モジュール引用 ; USE statements
  !

  ! netcdf データ入出力
  ! netcdf data input or output
  !
  use gtool_history

  ! 金星雲特性計算 (Pollack 1993)
  ! optical properties of Venus cloud (Pollack 1993)
  !
  use mod_pollackcloud, only: cloudlayerprop

  ! 内挿計算プログラム
  ! interpolation program
  !
  use interpol, only: PointInterPol

  ! 宣言文 ; Declaration statements
  !
  implicit none

  integer :: Flag_OUTPUT
                          ! 出力フォーマット指定フラグ
                          ! flag of output data file format
                          ! 1: netcdf file, 2: text file
  integer :: fnml = 12
                          ! 装置番号(name list file)
                          ! device number : namelist file
  character(99) :: InFilePress
                          ! 鉛直プロファイル入力ファイル名
                          ! input file name
  integer :: kmax, k_r
  real(8), allocatable :: r_Press(:)
  integer, allocatable :: z_LayNum(:)

  integer :: fi   = 11
  character(99) :: WaveNumData
  integer :: wmax, k_w
  real(8), allocatable :: w_WaveNum(:)

  integer :: fo   = 10
  character(99) :: OutFileParticle

  real(8), pointer :: a_Axis(:) => null()
                          ! 配列のサイズを調べるための配列
                          ! dummy array
  real(8) :: Rmed = 1.0d0
  real(8) :: Rre = 1.429D+0
  real(8) :: Rim = 1.420D-8

  integer :: pmax = 4
  integer :: k_p
  integer :: p_ParNum(1:4)
  real(8), allocatable :: zpw_DelOptDep  (:,:,:)
  real(8), allocatable :: zpw_SingleScatA(:,:,:)
  real(8), allocatable :: zpw_AsymFact   (:,:,:)

  character(99) :: FileRefIndex
  integer :: vmax = 154
  integer :: k_v
  real(8) :: v_H2SO4WaveNum(1:154)
  real(8) :: v_H2SO4RefReal(1:154)
  real(8) :: v_H2SO4RefImag(1:154)

  ! namelist file の設定
  ! setting of namelist file
  !
  namelist /initial_pollackcloud_nml/ &
    & InFilePress,     &
    & WaveNumData,     &
    & FileRefIndex,    &
    & Flag_OUTPUT,     &
    & OutFileParticle

  ! namelist file の読み込み
  ! read namelist file
  !
  open(fnml, file='dcrtm.nml')
  read(fnml, nml=initial_pollackcloud_nml)
  close(fnml)

  ! 設定値のチェック
  ! check setting values
  !
  print *, '== Setting Values =='
  print *, 'InFilePress:', InFilePress
  print *, 'WaveNumData:', WaveNumData
  print *, 'FileRefIndex:', FileRefIndex

  print *, 'Flag_OUTPUT:', Flag_OUTPUT
    if(Flag_OUTPUT == 1) then
      print *, '* output file is NETCDF format'
    elseif(Flag_OUTPUT == 2) then
      print *, '* output file is TEXT format'
    else
      print *, '* invalid value (Flag_OUTPUT = 1 or 2): stop'
      stop
    end if

  print *, 'OutFileParticle:', OutFileParticle

  ! 鉛直プロファイルの読み込み
  ! read vertical profile
  !
  ! 配列サイズを調べる
  ! check array size
  !
  call HistoryGetPointer(''//InFilePress//'', 'r_Press', a_Axis )
  kmax = size( a_Axis ) - 1
  Nullify( a_Axis )

  allocate( r_Press  (0:kmax) )
  allocate( z_LayNum (1:kmax) )

  call HistoryGet(''//InFilePress//'', 'r_Press', r_Press)

  do k_r = 0, kmax
    print *, r_Press(k_r)
  end do


  ! 波数データの読み込み
  ! read wavenumber data
  !
  call HistoryGet(''//FileRefIndex//'', 'w_WaveNum', v_H2SO4WaveNum)
  call HistoryGet(''//FileRefIndex//'', 'w_RefReal', v_H2SO4RefReal)
  call HistoryGet(''//FileRefIndex//'', 'w_RefImag', v_H2SO4RefImag)

  open(fi, file=WaveNumData)
  read(fi, *)
  read(fi, *) wmax
  read(fi, *)

  wmax = wmax -1
  print *, wmax

  allocate( w_WaveNum      (              0:wmax) )
  allocate( zpw_DelOptDep  (1:kmax,1:pmax,0:wmax) )
  allocate( zpw_SingleScatA(1:kmax,1:pmax,0:wmax) )
  allocate( zpw_AsymFact   (1:kmax,1:pmax,0:wmax) )

  do k_w = 0, wmax
    read(fi, *) w_WaveNum(k_w)
    print *, w_WaveNum(k_w)

    if( (w_WaveNum(k_w) > v_H2SO4WaveNum(vmax)) .or. &
      & (w_WaveNum(k_w) < v_H2SO4WaveNum(   1)) ) then
      print *, 'invalid wavenumber, wavenumber:', w_WaveNum(k_w)
      print *, v_H2SO4WaveNum(1), 'to', v_H2SO4WaveNum(vmax), '[m-1] is OK.'
      stop
    end if

  end do

  close(fi)

  ! 光学特性の計算
  ! calculate optical properties
  !
  do k_w = 0, wmax

    print *, 'wavenumber:', w_WaveNum(k_w)

    call PointInterPol(                          & ! 2017/01/26 modify
      & 1, vmax, v_H2SO4WaveNum, v_H2SO4RefReal, & ! (in )
      & w_WaveNum(k_w),                          & ! (in )
      & Rre                                      ) ! (out)

    print *, 'real part of refractive index:', Rre

    call PointInterPol(                          & ! 2017/01/26 modify
      & 1, vmax, v_H2SO4WaveNum, v_H2SO4RefImag, & ! (in )
      & w_WaveNum(k_w),                          & ! (in )
      & Rim                                      ) ! (out)

    print *, 'imaginary part of refractive index', Rim


    call cloudlayerprop (              &
      & kmax, r_Press, w_WaveNum(k_w), & ! (in )
      & Rmed, Rre, Rim,                & ! (in )
      & zpw_DelOptDep  (:,:,k_w),      & ! (out)
      & zpw_SingleScatA(:,:,k_w),      & ! (out)
      & zpw_AsymFact   (:,:,k_w)       ) ! (out)

  end do ! k_w

  ! データ出力
  ! data output
  !
  if(Flag_OUTPUT == 1) then

    do k_r = 1, kmax
      z_LayNum(k_r) = k_r
    end do

    do k_p = 1, pmax
      p_ParNum(k_p) = k_p
    end do

    call HistoryCreate(                      &
      & file=''//OutFileParticle//'',        &
      & title=''//OutFileParticle//'',       &
      & source='initial_pollackcloud.f90',       &
      & institution='DCRTM',                 &
      & dims=(/'r_Press  ','z_LayNum ','p_ParNum ','w_WaveNum'/), &
      & dimsizes=(/kmax+1,kmax,pmax,wmax+1/),   &
      & longnames=(/'pressure(boundary)',   &
      &             'layer number      ',   &
      &             'particle number   ',   &
      &             'wavenumber        '/), &
      & units=(/'Pa ', '1  ', '1  ', 'm-1'/)    )

    call HistoryPut('r_Press  ',r_Press)
    call HistoryPut('z_LayNum ',z_LayNum)
    call HistoryPut('p_ParNum ',p_ParNum)
    call HistoryPut('w_WaveNum',w_WaveNum)

    call HistoryAddVariable( &
      & varname='zpw_DelOptDep', dims=(/'z_LayNum ','p_ParNum ','w_WaveNum'/), &
      & longname='optical depth of layer (particle)', units='1', xtype='double')
    call HistoryPut('zpw_DelOptDep',zpw_DelOptDep)

    call HistoryAddVariable( &
      & varname='zpw_SingleScatA', dims=(/'z_LayNum ','p_ParNum ','w_WaveNum'/), &
      & longname='single scattering albedo (particle)', units='1', xtype='double')
    call HistoryPut('zpw_SingleScatA',zpw_SingleScatA)

    call HistoryAddVariable( &
      & varname='zpw_AsymFact', dims=(/'z_LayNum ','p_ParNum ','w_WaveNum'/), &
      & longname='asymmetry factor (particle)', units='1', xtype='double')
    call HistoryPut('zpw_AsymFact',zpw_AsymFact)

    call HistoryClose

  elseif(Flag_OUTPUT == 2) then

    open(fo, file=''//OutFileParticle//'')

    write(fo, *) '-- number of species, wavenumbers (>=2), layers (boundary-1)'
    write(fo, *) pmax, wmax+1, kmax !2  3  32
    write(fo, *) '-- boundary pressure [Pa] (layers + 1)'
    do k_r = 0, kmax
      write(fo, *) r_Press(k_r)
    end do

    do k_p = 1, pmax
      write(fo, *) '== species', k_p

      do k_w = 0, wmax
        write(fo, *) '-- wavenumber', k_w+1, '[m-1]'
        write(fo, *) '-- optical depth, single scattering albedo, asymmetry factor'

        write(fo, *) w_WaveNum(k_w)
        do k_r = 1, kmax
          write(fo, *) zpw_DelOptDep(k_r,k_p,k_w),   &
            &          zpw_SingleScatA(k_r,k_p,k_w), &
            &          zpw_AsymFact(k_r,k_p,k_w)
        end do ! k_r in k_w, k_p

      end do ! k_w in k_p

    end do ! k_p

    close(fo)

  else
    print *, 'invalid Flag_OUTPUT (1 or 2):', Flag_OUTPUT
    stop
  end if


end program initialdata_pollackcloud
